__AUTHOR__ = "Dipankar Pal"
__EMAIL__ = "dipankarpal5050@gmail.com"
__WEBSITE__ = "deep5050.github.io"

# def get_version():
#     return __VERSION__

# if __name__ == "__main__":
#     get_version()



"""
    Version of the current program, ( in development mode it needs to be updated in every realease)
"""
__VERSION__ = "2.1.2"


def get_version():
    """ get the version number as string """
    return __VERSION__


if __name__ == "__main__":
    get_version()



import argparse
import sys
from zenlog import log


class Parser:

    """Parse the command-line args and retrun result to the __main__"""

    def __init__(self):
        self.parser = None
        self.result = None

        self.parser = argparse.ArgumentParser(
            description="Play any radio around the globe right from the CLI ",
            prog="radio-active",
        )

        self.parser.add_argument(
            "--version", "-V", action="store_true", dest="version", default=False
        )

        self.parser.add_argument(
            "--station",
            "-S",
            action="store",
            dest="station_name",
            help="Specify a station name",
        )

        self.parser.add_argument(
            "--uuid",
            "-U",
            action="store",
            dest="station_uuid",
            help="Specify a station UUID",
        )

        self.parser.add_argument(
            "--log-level",
            "-L",
            action="store",
            default="info",
            dest="log_level",
            help="Specify log level",
        )

    def parse(self):
        self.result = self.parser.parse_args()

        if self.result is None:
            log.error("Could not parse the arguments properly")
            sys.exit(1)
        return self.result



""" FFplay proess handler """

import os
import sys
from shutil import which
from signal import SIGTERM
from subprocess import Popen
from time import sleep

import psutil
from zenlog import log


class Player:

    """FFPlayer handler, it holds all the attributes to properly execute ffplay
    FFmepg required to be installed seperately
    """

    def __init__(self, URL):
        self.url = URL
        self.is_playing = False
        self.process = None
        self.exe_path = None
        self.program_name = "ffplay"  # constant value

        log.debug("player: url => {}".format(self.url))
        # check if FFplay is installed
        self.exe_path = which(self.program_name)
        log.debug("FFplay: {}".format(self.exe_path))

        if self.exe_path is None:
            log.critical("FFplay not found, install it first please")
            sys.exit(1)

        self.process = Popen(
            [self.exe_path, "-nodisp", "-nostats", "-loglevel", "0", self.url],
            shell=False,
        )

        log.debug("player: ffplay => PID {} initiated".format(self.process.pid))

        sleep(3)  # sleeping for 3 seconds wainting for ffplay to start properly

        if self.is_active():
            self.is_playing = True
            log.info("Radio started successfully")
        else:
            log.error("Radio could not be stared, may be a dead station")
            sys.exit(0)

    def is_active(self):
        """checks for if the ffplay is still active or not,
        will be used to terminate FFPLAY when the radioactive terminates"""

        proc = psutil.Process(self.process.pid)
        if proc.status() == psutil.STATUS_ZOMBIE:
            return False
        return True

    def play(self):
        """Nothing"""
        if not self.is_playing:
            pass  # call the init function again ?

    def stop(self):
        """sends a SIGTERM to the process id of the current FFPLAY"""

        if self.is_playing:
            log.debug("Killing ffplay PID: {}".format(self.process.pid))
            os.kill(self.process.pid, SIGTERM)
        else:
            log.warn("Player: radio is not playing")



#!/usr/bin/env python
import signal
import sys

from zenlog import log

from radioactive.alias import Alias
from radioactive.app_version import get_version
from radioactive.args import Parser
from radioactive.handler import Handler
from radioactive.last_station import Last_station
from radioactive.player import Player

# globally needed as signal handler needs it
# to terminate main() properly
player = None

import sentry_sdk
sentry_sdk.init(
    "https://e3c430f3b03f49b6bd9e9d61e7b3dc37@o615507.ingest.sentry.io/5749950",

    # Set traces_sample_rate to 1.0 to capture 100%
    # of transactions for performance monitoring.
    # We recommend adjusting this value in production.
    traces_sample_rate=1.0,
    debug=True,
)

def main():
    parser = Parser()
    args = parser.parse()

    handler = Handler()

    alias = Alias()
    alias.generate_map()

    last_station = Last_station()

    mode_of_search = ""
    VERSION = get_version()

    if args.version:
        log.info("RADIO-ACTIVE : version {}".format(VERSION))
        sys.exit(0)

    station_name = args.station_name
    station_uuid = args.station_uuid
    log_level = args.log_level

    if log_level in ["info", "error", "debug"]:
        log.level(log_level)
    else:
        log.level("info")
        log.warning(
            "Correct log levels are: error,warning,info(default),debug")

    # -------------------- NOTHING PROVIDED --------------------- #
    # if neither of --station and --uuid provided , look in last_station file

    if station_name is None and station_uuid is None:
        # try to fetch the last played station's information
        log.warn(
            "No station information provided, trying to get the last station")
        # getting last station details, getting the UUID
        station_uuid = last_station.get_info()

    # ------------------------------------------------------------ #

    # --------------------ONLY UUID PROVIDED --------------------- #
    # if --uuid provided call directly

    if station_uuid is not None:
        mode_of_search = "uuid"

    # ------------------------------------------------------------ #

    # ------------------- ONLY STATION PROVIDED ------------------ #

    elif station_name is not None and station_uuid is None:
        # got station name only, looking in alias (if any)

        result = alias.search(station_name)
        if result is not None and alias.found:
            station_uuid = result["stationuuid"]

        if alias.found:
            mode_of_search = "uuid"
        else:
            log.debug("Alias not found, using normal API search")
            mode_of_search = "name"
    # ------------------------------------------------------------ #

    log.debug("Mode of search: {}".format(mode_of_search))

    if mode_of_search == "uuid":
        handler.play_by_station_uuid(station_uuid)

    else:
        handler.play_by_station_name(station_name)

    global player
    player = Player(handler.target_station["url"])

    # writing the station name to a file, next time if user
    # don't specify anything, it will try to start the last station
    last_station.save_info(handler.target_station)

    signal.pause()


def signal_handler(sig, frame):
    global player
    log.debug("You pressed Ctrl+C!")
    log.debug("Stopping the radio")
    if player.is_playing:
        player.stop()
    log.info("Exiting now")
    sys.exit(0)


signal.signal(signal.SIGINT, signal_handler)

if __name__ == "__main__":
    main()



""" This module saves the current playing station information to a hidden file,
and loads the data when no arguments are provide """

import json
import os.path
import sys
from zenlog import log


class Last_station:

    """ Saves the last played radio station information, when user don't provide any -S or -U
    it looks for the information.

    on every successful run, it saves the station information.
    The file it uses to store the data is a hidden file under users' home directory
    """

    def __init__(self):
        self.last_station_path = None

        self.last_station_path = os.path.join(
            os.path.expanduser("~"), ".radio-active-last-station.json")

    def get_info(self):
        try:
            with open(self.last_station_path, "r") as f:
                last_station = json.load(f)
                log.info("Playing last station: {}".format(
                    last_station["name"]))
                return last_station["stationuuid"]
        except Exception:
            log.critical(
                "Need a station name  or UUID to play the radio, see help")
            sys.exit(0)

    def save_info(self, station):
        """ dumps the current station information as a json file """

        log.debug("Dumping station information")
        with open(self.last_station_path, "w") as f:
            json.dump(station, f)



from setuptools import find_packages
from setuptools import setup

from radioactive.app_version import get_version

DESCRIPTION = "Play any radio around the globe right from the terminal"
VERSION = get_version()


def readme():
    with open("README.md") as f:
        return f.read()


def required():
    with open("requirements") as f:
        return f.read().splitlines()


setup(
    name="radio-active",
    version=VERSION,
    description=DESCRIPTION,
    long_description=readme(),
    long_description_content_type="text/markdown",
    keywords="pyradios wrapper radios api shortwave internet-radio cli app",
    author="Dipankar Pal",
    author_email="dipankarpal5050@gmail.com",
    url="https://github.com/deep5050/radio-active",
    license="MIT",
    entry_points={
        "console_scripts": [
            "radioactive = radioactive.__main__:main",
            "radio = radioactive.__main__:main",
        ]
    },
    packages=find_packages(),
    install_requires=required(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Multimedia :: Sound/Audio",
    ],
    python_requires=">=3.6",
    project_urls={
        "Source": "https://github.com/deep5050/radio-active/",
        "Upstream": "https://api.radio-browser.info/",
    },
)



import os.path

from zenlog import log


class Alias:
    def __init__(self):
        self.alias_map = []
        self.found = False

        self.alias_path = os.path.join(os.path.expanduser("~"),
                                       "radio-active-alias.txt")

    def generate_map(self):
        # create alias map
        if os.path.exists(self.alias_path):
            # log.debug("Alias file exists")
            try:
                with open(self.alias_path, "r") as f:
                    alias_data = f.read()
                    alias_list = alias_data.splitlines()
                    for alias in alias_list:
                        temp = alias.split("=")
                        left = temp[0]
                        right = temp[1]
                        self.alias_map.append({"name": left, "uuid": right})

                # log.debug(json.dumps(alias_map, indent=3))
            except Exception as e:
                log.warning("could not get / parse alias data")
            # log.debug(json.dumps(self.alias_map))
        else:
            log.warning("Alias file does not exist")

        # log.debug(json.dumps(self.alias_map, indent=3))

    def search(self, entry):
        if len(self.alias_map) > 0:
            log.debug("looking under alias file")
            for alias in self.alias_map:
                if alias["name"] == entry:
                    log.debug("Alias found: {} = {}".format(
                        alias["name"], alias["uuid"]))
                    self.found = True
                    return alias

            log.debug("Alias not found")
        else:
            log.debug("Empty Alias file")

        return None



"""
    This handler solely depends on pyradios module to communicate with our remote API
"""

import json
import sys

from pyradios import RadioBrowser
from zenlog import log


class Handler:
    """
    radio-browser API handler. This module communicates with the underlying API via PyRadios
    """

    def __init__(self):
        self.API = None
        self.response = None
        self.target_station = None

        # When RadioBrowser can not be initiated properly due to no internet (probably)
        try:
            self.API = RadioBrowser()
        except IndexError as e:
            log.critical("Something wrong with your internet connection")
            log.debug(e)
            sys.exit(1)

    def station_validator(self):
        """Validate a response from the API and takes appropriate decision"""

        # when no response from the API
        if not self.response:
            log.error("No stations found by the name")
            sys.exit(1)

        # when multiple results found
        if len(self.response) > 1:
            log.warn("{} stations found by the name".format(len(self.response)))

            for station in self.response:
                data = {}
                data["name"] = station["name"]
                data["uuid"] = station["stationuuid"]
                data["country"] = station["country"]
                log.info(json.dumps(data, indent=3))
            sys.exit(1)

        # when exactly one response found
        if len(self.response) == 1:
            log.info("Station found: {}".format(self.response[0]["name"]))
            log.debug(json.dumps(self.response[0], indent=3))
            self.target_station = self.response[0]
            self.API.click_counter(self.target_station["stationuuid"])

    def play_by_station_name(self, _name=None):
        """search and play a station by its name"""

        self.response = self.API.search(name=_name, name_exact=False)
        self.station_validator()

    def play_by_station_uuid(self, _uuid):
        """search and play station by its stationuuid"""

        # Pyradios by default don't let you search by uuid
        # a trick is to call click_counter(uuid) directly to get the station info
        is_ok = "false"
        try:
            self.target_station = self.API.click_counter(_uuid)
            log.debug(json.dumps(self.target_station, indent=3))
            is_ok = self.target_station["ok"]
        except Exception:
            log.error("Could not find the station by the UUID")
            sys.exit(0)

        log.info("Station found: {}".format(self.target_station["name"]))
        temp = self.API.search(name=self.target_station["name"], name_exact=True)
        log.debug(json.dumps(temp, indent=3))

        # register a valid click against the current response
        if is_ok == "false":
            res = self.API.click_counter(self.target_station["stationuuid"])
            log.debug(json.dumps(res, indent=3))
